import numpy as np
import networkx as nx
import argparse

parser = argparse.ArgumentParser()
parser.add_argument('-n', '--protno', type=int, default=1,help='Peptide number')
# Parse and print the results
args = parser.parse_args()
numpeptides = args.protno

#################THIS SEQUENCE###########

f = open("seq.txt", 'r')
A = f.readlines()
f.close()
string = A[0].strip()

Ar_resids = [pos for pos, char in enumerate(string) if char in 'FYWH']
Qd_resids = [pos for pos, char in enumerate(string) if char in 'KR']


#########################################




with open("martini_v2.P.itp") as f:
	content = f.readlines()
this = [x.strip() for x in content] 

initial = this[:2]
#divide into multiple parts
startpoint_test = np.where(np.array(this) == '[ atomtypes ]')[0][0]
endpoint_test = np.where(np.array(this) == '')[0][np.where(np.array(this) == '')[0] > startpoint_test].min()
atoms_test = this[startpoint_test+1:endpoint_test]
atomtype2 = np.array([i.split() for i in atoms_test])


startpoint_test = np.where(np.array(this) == '[ nonbond_params ]')[0][0]
endpoint_test = np.where(np.array(this) == '')[0][np.where(np.array(this) == '')[0] > startpoint_test].min()
atoms_test = this[startpoint_test+2:endpoint_test]
nb_self = np.array([i.split() for i in atoms_test], dtype=object)


nb_self = np.array([i[:5] for i in nb_self], dtype=object)
G1 = nx.Graph()
edgelist = nb_self[:,:2]
proplist = nb_self[:,3:5].astype(float)
Sigmas = [(i[1]/i[0])**(1.0/6) for i in proplist]
Epsilons = [(i[0]**2)/(4.0*i[1]) for i in proplist]

for i in range(len(edgelist)):
    G1.add_edge(edgelist[i][0], edgelist[i][1], weight=Epsilons[i], another=proplist[i])

############################# Now MARTINI is loaded into G1 #########################################
############################# Spare atomtypes are created for convinience -- cationic groups and aromatic groups #########################


G3 = nx.Graph()
#Atomtypes that are identical: C2, C3, C4, C5, SC2, SC3, SC4, SC5, Qa, Q0, Qd, N0, Nd, SNd, SN0
similar = ['POL', 'C1', 'C2', 'C3', 'C4', 'C5', 'SC1', 'SC2', 'SC3', 'SC4', 'SC5', 'Qa', 'Q0', 'Qd', 'N0', 'Nd', 'Nda', 'SNd', 'SN0']
for i in similar:
    for j in similar:
        G3.add_edge(i, j, weight=G1.edges[i, j]['weight'], another=G1.edges[i, j]['another'])

tillnow = list(G3.nodes)
Newadditions = ['PP5', 'PP4', 'PP1', 'PNa', 'PNda', 'PN0', 'PSP1', 'PSNd']
for i in Newadditions:
    for j in Newadditions:
        G3.add_edge(i, j, weight=G1.edges[i[1:], j[1:]]['weight'], another=G1.edges[i[1:], j[1:]]['another'])
        
for i in Newadditions:
    for j in similar:
        G3.add_edge(i, j, weight=G1.edges[i[1:], j]['weight'], another=G1.edges[i[1:], j]['another'])   
        
tillnow=list(G3.nodes) 

Newadditions = ['APP5']       
for i in Newadditions:
    for j in tillnow:
        G3.add_edge(i, j, weight=G3.edges['PP5', j]['weight'], another=G3.edges['PP5', j]['another'])   
G3.add_edge('APP5', 'APP5', weight=G3.edges['PP5', 'PP5']['weight'], another=G3.edges['PP5', 'PP5']['another'])        
tillnow=list(G3.nodes)     
 
 
newcharges = ['Qd' + str(i+1) for i in range(len(Qd_resids)*numpeptides)]       
Newadditions = ['AQd'] + newcharges
for i in Newadditions:
    for j in tillnow:
        G3.add_edge(i, j, weight=G3.edges['Qd', j]['weight'], another=G3.edges['Qd', j]['another'])
        
for i in Newadditions:
    for j in Newadditions:
        G3.add_edge(i, j, weight=G3.edges['Qd', j]['weight'], another=G3.edges['Qd', j]['another'])
        

#Change C1-POL Make this more hydrophobic
epsilon = 1.0000008
sigma = 0.47
c6 = 4*epsilon*((sigma)**6)
c12 = 4*epsilon*((sigma)**12)
G3.edges['C1', 'POL']['weight'] = epsilon
G3.edges['C1', 'POL']['another'] = np.array([c6, c12])
#Change SC1-POL Make this more hydrophobic
epsilon = 1.0000008
sigma = 0.47
c6 = 4*epsilon*((sigma)**6)
c12 = 4*epsilon*((sigma)**12)
G3.edges['SC1', 'POL']['weight'] = epsilon
G3.edges['SC1', 'POL']['another'] = np.array([c6, c12])


tillnow=list(G3.nodes) 

Aro = ['AR'+str(i+1) for i in range(len(Ar_resids)*numpeptides)] 
AroPol = ['APR'+str(i+1) for i in range(len(Ar_resids)*numpeptides)]    
Newadditions = ['APC1'] + Aro + AroPol
for i in Newadditions:
    for j in tillnow:
        G3.add_edge(i, j, weight=G3.edges['SC1', j]['weight'], another=G3.edges['SC1', j]['another']) 
        
for i in Newadditions:
    for j in Newadditions:
        G3.add_edge(i, j, weight=G3.edges['SC1', j]['weight'], another=G3.edges['SC1', j]['another'])        
tillnow=list(G3.nodes)



tillnow=list(G3.nodes)     
Newadditions = ['PAC1']
for i in Newadditions:
    for j in tillnow:
        G3.add_edge(i, j, weight=G3.edges['C1', j]['weight'], another=G3.edges['C1', j]['another']) 
        
for i in Newadditions:
    for j in Newadditions:
        G3.add_edge(i, j, weight=G3.edges['C1', j]['weight'], another=G3.edges['C1', j]['another'])        
tillnow=list(G3.nodes)


#Change AR*-POL
AR =  Aro + AroPol
epsilon = 1.5
sigma = 0.47
c6 = 4*epsilon*((sigma)**6)
c12 = 4*epsilon*((sigma)**12)
for i in AR:
    G3.edges[i, 'POL']['weight'] = epsilon
    G3.edges[i, 'POL']['another'] = np.array([c6, c12])

#Add AR*-APP5 interaction epsilon: 3.5   
AR =  Aro + AroPol
epsilon = 3.5
sigma = 0.47
c6 = 4*epsilon*((sigma)**6)
c12 = 4*epsilon*((sigma)**12)
for i in AR:
    G3.edges[i, 'APP5']['weight'] = epsilon
    G3.edges[i, 'APP5']['another'] = np.array([c6, c12])
 
#Polar-Polar Group Reductions:
#Check these edits!
Polargrps = ['APP5', 'PP5', 'PP4', 'PP1', 'PNa', 'PNda', 'PN0', 'PSP1', 'PSNd'] + ['APC1'] + AroPol
Chargedgrps = ['Qa', 'Q0', 'Qd', 'AQd'] + newcharges
water = ['POL']

smaller_radii = ['PSP1', 'PSNd'] + AroPol 

for i in Polargrps:
    for j in Polargrps:
        if (i != j):
            epsilon = G3.edges[i, j]['weight'] - 0.29 #Due to double counting, the value comes to 0.58, From previous papers/work
        else:
            epsilon = G3.edges[i, j]['weight'] - 0.58 #No double counting here.
            
            
        if ((i in smaller_radii) and (j in smaller_radii)):
            sigma=0.43
        else:
            sigma=0.47
        c6 = 4*epsilon*(sigma**6)
        c12 = 4*epsilon*(sigma**12)
        G3.edges[i, j]['weight'] = epsilon
        G3.edges[i, j]['another'] = np.array([c6, c12])


        
#Polar-POL adjustments
for i in water:
    for j in Polargrps:
        epsilon = G3.edges[i, j]['weight'] - 0.58 #From previous papers/work
        sigma=0.47
        c6 = 4*epsilon*(sigma**6)
        c12 = 4*epsilon*(sigma**12)
        G3.edges[i, j]['weight'] = epsilon
        G3.edges[i, j]['another'] = np.array([c6, c12])

        
#Polar-Charged Group Reductions:

for i in Polargrps:
    for j in Chargedgrps:
        epsilon = G3.edges[i, j]['weight'] - 0.28 #From previous papers/work
        sigma=0.47
        c6 = 4*epsilon*(sigma**6)
        c12 = 4*epsilon*(sigma**12)
        G3.edges[i, j]['weight'] = epsilon
        G3.edges[i, j]['another'] = np.array([c6, c12])


######DUMMIES########

tillnow = list(G3.nodes)
#D1, D1A
D1f = ['D1', 'D1A']
c6 = 0.0
c12 = 4.5387e-10

for i in tillnow:
    for j in D1f:
        G3.add_edge(i, j, weight=0.0, another=np.array([c6, c12]))

for i in D1f:
    for j in D1f:
        G3.add_edge(i, j, weight=0.0, another=np.array([c6, c12]))

tillnow = list(G3.nodes)
D2f = ['D2', 'D2A']
c6 = 0.0
c12 = 4.5387e-10
for i in tillnow:
    for j in D2f:
        G3.add_edge(i, j, weight=0.0, another=np.array([c6, c12]))
        
for i in D2f:
    for j in D2f:
        G3.add_edge(i, j, weight=0.0, another=np.array([c6, c12]))
        
special = Chargedgrps
for i in special:
    G3.edges[i, 'D2']['another'] = np.array([c6, 4.5387e-07]) 
    G3.edges[i, 'D2A']['another'] = np.array([c6, 4.5387e-07])      

tillnow = list(G3.nodes)
Df = ['D']
c6 = 0.0
c12 = 0.0
for i in tillnow:
    for j in Df:
        if i not in D1f+D2f:
            G3.add_edge(i, j, weight=0.0, another=np.array([c6, c12]))
        else:
            G3.add_edge(i, j, weight=0.0, another=np.array([c6, 4.5387e-10]))
G3.add_edge('D', 'D', weight=0.0, another=np.array([c6, c12]))



#Add Cation Pi Here


Relevant = []
for i in range(len(Qd_resids)):
    for j in range(len(Ar_resids)):
        if np.abs(Qd_resids[i] - Ar_resids[j]) > 2:
            S = [[i+(len(Qd_resids)*p), j+(len(Ar_resids)*p)] for p in range(numpeptides)]
            for t in S:
                Relevant.append( [t[0], t[1]] )

Relevant = np.array(Relevant)+1

epsilon = 3.0 #Change Here
sigma = 0.47
c6 = 4*epsilon*((sigma)**6)
c12 = 4*epsilon*((sigma)**12) 

for i in range(len(Relevant)):
    G3.edges['Qd'+str(Relevant[i][0]), 'AR'+str(Relevant[i][1])]['weight'] = epsilon
    G3.edges['Qd'+str(Relevant[i][0]), 'AR'+str(Relevant[i][1])]['another'] = np.array([c6, c12])
    
epsilon = 3.0-0.28 #Change Here
sigma = 0.47
c6 = 4*epsilon*((sigma)**6)
c12 = 4*epsilon*((sigma)**12) 

for i in range(len(Relevant)):
    G3.edges['Qd'+str(Relevant[i][0]), 'APR'+str(Relevant[i][1])]['weight'] = epsilon
    G3.edges['Qd'+str(Relevant[i][0]), 'APR'+str(Relevant[i][1])]['another'] = np.array([c6, c12])

tillnow = list(G3.nodes)



#Specific_Cation_Pi_ADDED: Oct_7_2021
Spgrps = ['Q0L', 'QdN']
for i in tillnow:
    for j in Spgrps:
        G3.add_edge(i, j, weight=G3.edges[i, j[:-1]]['weight'], another=G3.edges[i, j[:-1]]['another'])
for i in Spgrps:
    for j in Spgrps:
        G3.add_edge(i, j, weight=G3.edges[i[:-1], j[:-1]]['weight'], another=G3.edges[i[:-1], j[:-1]]['another'])


epsilon = 3.0 #Change Here
sigma = 0.47
c6 = 4*epsilon*((sigma)**6)
c12 = 4*epsilon*((sigma)**12) 

for i in Aro:
    G3.edges['Q0L', i]['weight'] = epsilon
    G3.edges['Q0L', i]['another'] = np.array([c6, c12])
    G3.edges['QdN', i]['weight'] = epsilon
    G3.edges['QdN', i]['another'] = np.array([c6, c12])
    
epsilon = 3.0-0.28 #Change Here
sigma = 0.47
c6 = 4*epsilon*((sigma)**6)
c12 = 4*epsilon*((sigma)**12) 

for i in AroPol:
    G3.edges['Q0L', i]['weight'] = epsilon
    G3.edges['Q0L', i]['another'] = np.array([c6, c12])
    G3.edges['QdN', i]['weight'] = epsilon
    G3.edges['QdN', i]['another'] = np.array([c6, c12])








####Introduce Lower Leaflet #####
tillnow = list(G3.nodes)
Lower = ['PNa_L', 'Qa_L', 'Q0_L']

for i in tillnow:
    for j in Lower:
        G3.add_edge(i, j, weight=G3.edges[i, j[:-2]]['weight'], another=G3.edges[i, j[:-2]]['another'])

for i in Lower:
    for j in Lower:
        G3.add_edge(i, j, weight=G3.edges[i[:-2], j[:-2]]['weight'], another=G3.edges[i[:-2], j[:-2]]['another'])

#Reduction
for i in Lower:
    G3.edges[i, 'PP5']['another'] = np.array([0.0, 0.00092953]) 





ff = open("proMPTff.itp", "w+")
ff.writelines("[ defaults ]"+"\n")
ff.writelines("1 1 no 0.0 0.0"+"\n")
ff.writelines("\n")
ff.writelines("[ atomtypes ]"+"\n")
ff.writelines("\n")

#Fullsized
FS = ['C1', 'C2', 'C3', 'C4', 'C5', 'Qa', 'Q0', 'Qd', 'AQd', 'PAC1', 'N0', 'Nd', 'Nda', 'Qa_L', 'Q0_L', 'Q0L', 'QdN'] + newcharges
for i in FS:
    ff.writelines(i + "\t" +str(72.00) + "\t" + "0.00   A   0.0   0.0"+"\n")
#Smallsized
SS = ['AC1', 'SC1', 'SC2', 'SC3', 'SC4', 'SC5', 'SNd', 'SN0']+Aro
for i in SS:
    ff.writelines(i + "\t" +str(45.00) + "\t" + "0.00   A   0.0   0.0"+"\n")
#Fullpolar
FP = ['APP5', 'PP5', 'PP4', 'PP1', 'PNa', 'PNda', 'PN0', 'POL', 'PNa_L']
for i in FP:
    ff.writelines(i + "\t" +str(24.00) + "\t" + "0.00   A   0.0   0.0"+"\n")
#Smallpolar
SP = ['PSP1', 'PSNd','APC1'] + AroPol
for i in SP:
    ff.writelines(i + "\t" +str(15.00) + "\t" + "0.00   A   0.0   0.0"+"\n")
#Dummies
Dum1 = ['D1', 'D1A', 'D2', 'D']
for i in Dum1:
    ff.writelines(i + "\t" +str(24.00) + "\t" + "0.00   A   0.0   0.0"+"\n")
Dum2 = ['D2A']
for i in Dum2:
    ff.writelines(i + "\t" +str(15.00) + "\t" + "0.00   A   0.0   0.0"+"\n")

ff.writelines("\n")    
ff.write("[ nonbond_params ]")
ff.write("\n")
ff.write('; interactions\n')
EL = list(G3.edges)
for edgei in EL:
    c6 = G3.edges[edgei[0], edgei[1]]['another'][0]
    c12 = G3.edges[edgei[0], edgei[1]]['another'][1]
    ff.write(edgei[0] + '\t' + edgei[1] + '\t' + '1' + '\t'+ G3.edges[edgei[0], edgei[1]]['another'].astype(str)[0] + '\t' + G3.edges[edgei[0], edgei[1]]['another'].astype(str)[1] + " ; "+ str(round((c6**2)/(4.0*c12), 2))+ "  " + str(round((c12/c6)**(1.0/6), 2)) + '\n')
ff.write('\n')
ff.close()
